/*
 * nigpibtcl.c
 *
 * Tcl/Tk interface code to allow Tcl to talk to to the National Instruments
 * driver.
 *
 *   nigpib.c - Tcl extension to interface with National Instruments GPIB
 *   Copyright (C) 1999  Bill Krimmel - bill@resuba.com
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *  Bill Krimmel (bill@resuba.com)
 */

#include <tcl.h>
#include "sys/ugpib.h"

/* Debugging defines for ibdebug command - not a supplied national command */
#define DEBUG		0
#define CNTRL_IO 	1
#define DATA_WRITE	2
#define DATA_READ	4

typedef struct {
   char *name;
   int (*proc)();
} CmdInfo;

typedef struct {
   int errCode;
   char *err;
   char *desc;
} ErrInfo;

typedef struct {
   char *name;
   int code;
} OptInfo;

extern int errno;
static long debug = DEBUG;

static Tcl_HashTable cmdTable;
static Tcl_HashTable errTable;
static Tcl_HashTable optTable;

int (*CmdFind())();
void Mapstatustoascii (Tcl_Obj *resultPtr, int status);



int ibaskCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibbnaCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibcacCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibclrCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibcmdCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibcmdaCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibdiagCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibdebugCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibconfigCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibdevCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibdmaCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibeosCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibeotCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibgtsCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibistCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
/* int ibeventCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]); */ 
int ibfindCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int iblinesCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int iblloCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int iblnCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int iblocCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibonlCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibpadCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibpctCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibpokeCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibppcCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibrdCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibrdfCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibrppCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibrscCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibrspCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibrsvCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibsadCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibsicCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibsreCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibstopCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibtmoCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
/* int ibtrapCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]); */
int ibtrgCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibwaitCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibwrtCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ibwrtfCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
/* 488.2 commands */
int AllSpollCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int SendIFCCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int DevClearCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int DevClearListCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int EnableLocalCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int EnableRemoteCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int FindLstnCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int FindRQSCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int PPollCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int PPollConfigCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int PPollUnconfigCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int PassControlCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int RcvRespMsgCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ReadStatusByteCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ReceiveCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ReceiveSetupCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int ResetSysCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int SendCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int SendCmdsCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
int SendDataBytesCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);

int MapStatusToAsciiCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);



static CmdInfo commands[] =  {
   {"ibask",	ibaskCmd},
   {"ibbna",	ibbnaCmd},
   {"ibcac",	ibcacCmd},
   {"ibclr",	ibclrCmd},
   {"ibcmd",	ibcmdCmd},
   {"ibcmda",	ibcmdaCmd},
   {"ibdiag",	ibdiagCmd},
   {"ibconfig",	ibconfigCmd},
   {"ibdev",	ibdevCmd},
   {"ibdma",	ibdmaCmd},
   {"ibeos",	ibeosCmd},
   {"ibeot",	ibeotCmd},
   {"ibgts",	ibgtsCmd},
   {"ibist",	ibistCmd},
/*   {"ibevent",	ibeventCmd},  */
   {"ibfind",	ibfindCmd},
   {"iblines",	iblinesCmd},
   {"ibllo",	iblloCmd},
   {"ibln",	iblnCmd},
   {"ibloc",	iblocCmd},
   {"ibonl",	ibonlCmd},
   {"ibpad",	ibpadCmd},
   {"ibpct",	ibpctCmd},
   {"ibpoke",	ibpokeCmd},
   {"ibppc",	ibppcCmd},
   {"ibrd",	ibrdCmd},
   {"ibrdf",	ibrdfCmd},
   {"ibrpp",	ibrppCmd},
   {"ibrsc",	ibrscCmd},
   {"ibrsp",	ibrspCmd},
   {"ibrsv",	ibrsvCmd},
   {"ibsad",	ibsadCmd},
   {"ibsic",	ibsicCmd},
   {"ibsre",	ibsreCmd},
   {"ibstop",	ibstopCmd},
   {"ibtmo",	ibtmoCmd},
/* {"ibtrap",	ibtrapCmd}, */
   {"ibtrg",	ibtrgCmd},
   {"ibwait",	ibwaitCmd},
   {"ibwrt",	ibwrtCmd},
   {"ibwrtf",	ibwrtfCmd},
/* 488.2 commands */
   {"AllSpoll",		AllSpollCmd},
   {"SendIFC",		SendIFCCmd},
   {"DevClear",		DevClearCmd},
   {"DevClearList",	DevClearListCmd},
   {"FindLstn",		FindLstnCmd},
   {"EnableLocal",	EnableLocalCmd},
   {"EnableRemote",	EnableRemoteCmd},
   {"FindRQS",		FindRQSCmd},
   {"PPoll",		PPollCmd},
   {"PPollConfig",	PPollConfigCmd},
   {"PPollUnconfig",	PPollUnconfigCmd},
   {"PassControl",	PassControlCmd},
   {"RcvRespMsg",	RcvRespMsgCmd},
   {"ReadStatusByte",	ReadStatusByteCmd},
   {"Receive",		ReceiveCmd},
   {"ReceiveSetup",	ReceiveSetupCmd},
   {"ResetSys",		ResetSysCmd},
   {"Send",		SendCmd},
   {"SendCmds",		SendCmdsCmd},
   {"SendDataBytes",	SendDataBytesCmd},
/* Krimmel commands - just stuff to make life easier */
   {"ibdebug",		ibdebugCmd},
   {"MapStatusToAscii",	MapStatusToAsciiCmd},
/*   {"",		Cmd},  */
   { (char *) NULL, (int (*) ()) NULL}
};

static ErrInfo errors[] =  {
   {EDVR,	"EDVR",		"System error"},
   {ECIC,	"ECIC",		"Not CIC (or lost CIC during command)"},
   {ENOL,	"ENOL",		"Write detected and no listeners"},
   {EADR,	"EADR",		"Board not addressed correctly"},
   {EARG,	"EARG",		"Bad argument to function call"},
   {ESAC,	"ESAC",		"Function requires board to be SAC"},
   {EABO,	"EABO",		"Asynchronous operation was aborted"},
   {ENEB,	"ENEB",		"Non-existent board"},
   {EDMA,	"EDMA"		"DMA hardware error detected"},
   {EBTO,	"EBTO",		"DMA hardware uP bus timeout"},
   {EOIP,	"EOIP",		"New I/O attempted with old I/O in progress"},
   {ECAP,	"ECAP",		"No capability for intended opeation"},
   {EFSO,	"EFSO",		"File system operation error"},
   {EOWN,	"EOWN"		"Shareable board exclusively owned"},
   {EBUS,	"EBUS",		"Bus error"},
   {ESTB,	"ESTB",		"Serial poll queue overflow"},
   {ESRQ,	"ESRQ",		"SRQ line 'stuck' on, owwwwwch"},
   {ETAB,	"ETAB",		"The return buffer is full"},
   {ELCK,	"ELCK",		"Board or address is locked, lost combination"},
   {-1,		(char *) NULL, 	(char *) NULL}
};

static OptInfo options[] = {
   {"IbaPAD",			IbaPAD},
   {"IbaSAD",			IbaSAD},
   {"IbaTMO",			IbaTMO},
   {"IbaEOT",			IbaEOT},
   {"IbaPPC",			IbaPPC},
   {"IbaREADDR",		IbaREADDR},
   {"IbaAUTOPOLL",		IbaAUTOPOLL},
   {"IbaCICPROT",		IbaCICPROT},
   {"IbaIRQ",			IbaIRQ},
   {"IbaSC",			IbaSC},
   {"IbaSRE",			IbaSRE},
   {"IbaEOSrd",			IbaEOSrd},
   {"IbaEOSwrt",		IbaEOSwrt},
   {"IbaEOScmp",		IbaEOScmp},
   {"IbaEOSchar",		IbaEOSchar},
   {"IbaPP2",			IbaPP2},
   {"IbaTIMING",		IbaTIMING},
   {"IbaDMA",			IbaDMA},
   {"IbaReadAdjust",		IbaReadAdjust},
   {"IbaWriteAdjust",		IbaWriteAdjust},
   {"IbaEventQueue",		IbaEventQueue},
   {"IbaSPollBit",		IbaSPollBit},
   {"IbaSpollBit",		IbaSpollBit},
   {"IbaSendLLO",		IbaSendLLO},
   {"IbaSPollTime",		IbaSPollTime},
   {"IbaPPollTome",		IbaPPollTime},
   {"IbaNoEndBitOnEOS",		IbaNoEndBitOnEOS},
   {"IbaEndBitIsNormal",	IbaEndBitIsNormal},
   {"IbaUnAddr",		IbaUnAddr},
   {"IbaSignalNumber",		IbaSignalNumber},
   {"IbaHSCableLength",		IbaHSCableLength},
   {"IbaLON",			IbaLON},
   {"IbaBNA",			IbaBNA},
   {"IbaBaseAddr",		IbaBaseAddr},
   {"IbaDmaChannel",		IbaDmaChannel},
   {"IbaIrqLevel",		IbaIrqLevel},
   {"IbaBaud",			IbaBaud},
   {"IbaParity",		IbaParity},
   {"IbaStopBits",		IbaStopBits},
   {"IbaDataBits",		IbaDataBits},
   {"IbaComPort",		IbaComPort},
   {"IbaComIrqLevel",		IbaComIrqLevel},
   {"IbaComPortBase",		IbaComPortBase},
   {"IbaSingleCycleDma",	IbaSingleCycleDma},
   {"IbcPAD",			IbcPAD},
   {"IbcSAD",			IbcSAD},
   {"IbcTMO",			IbcTMO},
   {"IbcEOT",			IbcEOT},
   {"IbcPPC",			IbcPPC},
   {"IbcREADDR",		IbcREADDR},
   {"IbcAUTOPOLL",		IbcAUTOPOLL},
   {"IbcCICPROT",		IbcCICPROT},
   {"IbcIRQ",			IbcIRQ},
   {"IbcSC",			IbcSC},
   {"IbcSRE",			IbcSRE},
   {"IbcEOSrd",			IbcEOSrd},
   {"IbcEOSwrt",		IbcEOSwrt},
   {"IbcEOScmp",		IbcEOScmp},
   {"IbcEOSchar",		IbcEOSchar},
   {"IbcPP2",			IbcPP2},
   {"IbcTIMING",		IbcTIMING},
   {"IbcDMA",			IbcDMA},
   {"IbcReadAdjust",		IbcReadAdjust},
   {"IbcWriteAdjust",		IbcWriteAdjust},
   {"IbcEventQueue",		IbcEventQueue},
   {"IbcSPollBit",		IbcSPollBit},
   {"IbcSpollBit",		IbcSpollBit},
   {"IbcSendLLO",		IbcSendLLO},
   {"IbcSPollTime",		IbcSPollTime},
   {"IbcPPollTome",		IbcPPollTime},
   {"IbcNoEndBitOnEOS",		IbcNoEndBitOnEOS},
   {"IbcEndBitIsNormal",	IbcEndBitIsNormal},
   {"IbcUnAddr",		IbcUnAddr},
   {"IbcSignalNumber",		IbcSignalNumber},
   {"IbcHSCableLength",		IbcHSCableLength},
   {"IbcLON",			IbcLON},
   { (char *) NULL, 		0}
};

int gpibError (Tcl_Interp *interp, Tcl_Obj *resultPtr, char *string);

int Nigpib_Init ( Tcl_Interp *interp)  {
   CmdInfo *cmd;
   ErrInfo *err;
   OptInfo *opt;

   extern int gpibCmd	_ANSI_ARGS_(( ClientData clientData,
                              Tcl_Interp *interp,
                               int objc,
                               Tcl_Obj *CONST objv[]
                               ));


   Tcl_PkgProvide(interp, "gpib","0.1");

   Tcl_InitHashTable(&errTable, 1);
   for (err = errors; err->errCode > -1; err++)  {
      int newEntry;
      Tcl_HashEntry *entryPtr;
      entryPtr = Tcl_CreateHashEntry(&errTable, (char *)err->errCode, &newEntry);
      Tcl_SetHashValue(entryPtr, err->err);
   }

   Tcl_InitHashTable(&optTable, TCL_STRING_KEYS);
   for (opt = options; opt->name != NULL; opt++)  {
      int newEntry;
      Tcl_HashEntry *entryPtr;
      entryPtr = Tcl_CreateHashEntry(&optTable, opt->name, &newEntry);
      Tcl_SetHashValue(entryPtr, opt->code);
   }

   Tcl_InitHashTable(&cmdTable, TCL_STRING_KEYS);
   for (cmd = commands; cmd->name != NULL; cmd++)  {
      int newEntry;
      Tcl_HashEntry *entryPtr;
      entryPtr = Tcl_CreateHashEntry(&cmdTable, cmd->name, &newEntry);
      Tcl_SetHashValue(entryPtr, cmd->proc);
   }

   Tcl_CreateObjCommand(interp,"gpib" ,gpibCmd,
                         (ClientData) NULL,
                         (Tcl_CmdDeleteProc *) NULL );

   Tcl_ObjSetVar2(interp, Tcl_NewStringObj("ibsta", -1), NULL, Tcl_NewIntObj(ibsta), TCL_GLOBAL_ONLY);
   Tcl_ObjSetVar2(interp, Tcl_NewStringObj("iberr", -1), NULL, Tcl_NewIntObj(iberr), TCL_GLOBAL_ONLY);
   Tcl_ObjSetVar2(interp, Tcl_NewStringObj("ibcnt", -1), NULL, Tcl_NewIntObj(ibcnt), TCL_GLOBAL_ONLY);
   Tcl_ObjSetVar2(interp, Tcl_NewStringObj("ibcntl", -1), NULL, Tcl_NewLongObj(ibcntl), TCL_GLOBAL_ONLY);
    return TCL_OK;
}
/************************************************************************/
int gpibCmd(ClientData clientData, Tcl_Interp *interp,
      int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   Tcl_HashEntry *entryPtr;
   char *command;
   int retval;
   int (*handler)();

   resultPtr = Tcl_GetObjResult(interp);
   command = Tcl_GetStringFromObj(objv[1], NULL);

   entryPtr = Tcl_FindHashEntry(&cmdTable, command);
   if (entryPtr == NULL)  {
      Tcl_SetStringObj(resultPtr, "GPIB unknown command: ", -1);
      Tcl_AppendToObj(resultPtr, command, -1);
      return TCL_ERROR;
   }
   handler = (int(*)()) Tcl_GetHashValue(entryPtr);
   if (handler == NULL)  {
      Tcl_SetStringObj(resultPtr, "GPIB unknown command: ", -1);
      Tcl_AppendToObj(resultPtr, command, -1);
      return TCL_ERROR;
   }
   if (debug & CNTRL_IO)  {
      int i;
      printf("ibdebug - objc=%i\n", objc);
      for (i=0; i<objc; i++)  
         printf("objv[%i]=%s\n", i, Tcl_GetStringFromObj(objv[i], NULL));
      printf("===========================\n");
   }
   retval = (*handler) (clientData, interp, objc, objv);
   if (debug & CNTRL_IO)  {
      printf("Returned %i\n", retval);
      printf("===========================\n");
   }


   Tcl_ObjSetVar2(interp, Tcl_NewStringObj("ibsta", -1), NULL, Tcl_NewIntObj(ibsta), TCL_GLOBAL_ONLY);
   Tcl_ObjSetVar2(interp, Tcl_NewStringObj("iberr", -1), NULL, Tcl_NewIntObj(iberr), TCL_GLOBAL_ONLY);
   Tcl_ObjSetVar2(interp, Tcl_NewStringObj("ibcnt", -1), NULL, Tcl_NewIntObj(ibcnt), TCL_GLOBAL_ONLY);
   Tcl_ObjSetVar2(interp, Tcl_NewStringObj("ibcntl", -1), NULL, Tcl_NewLongObj(ibcntl), TCL_GLOBAL_ONLY);
   return retval;
}

/************************************************************************/

int ibaskCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, option, value;
   char *string;
   Tcl_HashEntry *entryPtr;

   resultPtr = Tcl_GetObjResult(interp);
   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud option");
      return TCL_ERROR;
   }

   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &option) != TCL_OK)  {  /* First check for numeric code */
      string = Tcl_GetStringFromObj(objv[3], NULL);
      entryPtr = Tcl_FindHashEntry(&optTable, string);
      if (entryPtr == NULL)  {
         Tcl_SetStringObj(resultPtr, "ibask: unknown option: ", -1);
         Tcl_AppendToObj(resultPtr, string, -1);
         return TCL_ERROR;
      }
      option = (int)Tcl_GetHashValue(entryPtr);
   }

   ibask(ud, option, &value);
   if (gpibError(interp, resultPtr, "ibask") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, value);
   return TCL_OK;
}

int ibbnaCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;
   char *name;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud name");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   name = Tcl_GetStringFromObj(objv[3], NULL);

   ibbna(ud, name);
   if (gpibError(interp, resultPtr, "ibbna") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, iberr);   /* return pervious board index */
   return TCL_OK;
}

int ibcacCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, sync;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetBooleanFromObj(interp, objv[3], &sync) != TCL_OK)
      return TCL_ERROR;

   ibcac(ud, sync);
   if (gpibError(interp, resultPtr, "ibcac") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibclrCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;


   ibclr(ud);
   if (gpibError(interp, resultPtr, "ibclr") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibcmdCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;
   long count;
   char *buf;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 5)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud, cmdbuf, cnt");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetLongFromObj(interp, objv[4], &count) != TCL_OK)
      return TCL_ERROR;

   buf = Tcl_GetStringFromObj(objv[3], NULL);

   ibcmd(ud, buf, count);
   if (gpibError(interp, resultPtr, "ibcmd") != TCL_OK)
      return TCL_ERROR;
   
   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibcmdaCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;
   long count;
   char *buf;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 5)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud, cmdbuf, cnt");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetLongFromObj(interp, objv[4], &count) != TCL_OK)
      return TCL_ERROR;

   buf = Tcl_GetStringFromObj(objv[3], NULL);

   ibcmda(ud, buf, count);
   if (gpibError(interp, resultPtr, "ibcmda") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibdiagCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;
   long count;
   char *buf;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 5)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud, cmdbuf, cnt");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetLongFromObj(interp, objv[4], &count) != TCL_OK)
      return TCL_ERROR;

   buf = Tcl_GetStringFromObj(objv[3], NULL);

   ibdiag(ud, buf, count);
   if (gpibError(interp, resultPtr, "ibdiag") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibdebugCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc < 2 && objc > 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "?debugLevel?");
      return TCL_ERROR;
   }
   if (objc == 3)  {
      if (Tcl_GetLongFromObj(interp, objv[2], &debug) != TCL_OK)
         return TCL_ERROR;
   }
   Tcl_SetLongObj(resultPtr, debug);   /* return status */
   return TCL_OK;
}

int ibconfigCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, option, value;
   char *string;
   Tcl_HashEntry *entryPtr;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 5)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud option value");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &option) != TCL_OK)  {
      string = Tcl_GetStringFromObj(objv[3], NULL);
      entryPtr = Tcl_FindHashEntry(&optTable, string);
      if (entryPtr == NULL)  {
         Tcl_SetStringObj(resultPtr, "ibask: unknown option: ", -1);
         Tcl_AppendToObj(resultPtr, string, -1);
         return TCL_ERROR;
      }
      option = (int)Tcl_GetHashValue(entryPtr);
   }

   if (Tcl_GetIntFromObj(interp, objv[4], &value) != TCL_OK)
      return TCL_ERROR;

   ibconfig(ud, option, value);
   if (gpibError(interp, resultPtr, "ibconfig") != TCL_OK)
      return TCL_ERROR;
   
   Tcl_SetIntObj(resultPtr, ibsta);
   return TCL_OK;
}

int ibdevCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int BdIndex, pad, sad, tmo, eot, eos, ud;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 8)  {
      Tcl_WrongNumArgs(interp, 1, objv, "BdIndex pad sad tmo eos");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &BdIndex) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &pad) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[4], &sad) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[5], &tmo) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetBooleanFromObj(interp, objv[6], &eot) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[7], &eos) != TCL_OK)
      return TCL_ERROR;

   ud = ibdev(BdIndex, pad, sad, tmo, eot, eos);
   if (gpibError(interp, resultPtr, "ibdev") != TCL_OK)
      return TCL_ERROR;
   if (ud == -1)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ud);
   return TCL_OK;
}

int ibdmaCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetBooleanFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibdma(ud, v);
   if (gpibError(interp, resultPtr, "ibdma") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibeosCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibeos(ud, v);
   if (gpibError(interp, resultPtr, "ibeos") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}


int ibeotCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetBooleanFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibeot(ud, v);
   if (gpibError(interp, resultPtr, "ibeot") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibgtsCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetBooleanFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibgts(ud, v);
   if (gpibError(interp, resultPtr, "ibgts") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibistCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetBooleanFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibist(ud, v);
   if (gpibError(interp, resultPtr, "ibist") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

/**************************************************************
*** this command seems to be missing from the Linux drivers ***
int ibeventCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, option;
   short event;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   ibevent(ud, &event); 
   if (gpibError(interp, resultPtr, "ibevent") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, event);
   return TCL_OK;
}
**************************************************************/
int ibfindCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;
   char *udname;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "udname");
      return TCL_ERROR;
   }
   udname = Tcl_GetStringFromObj(objv[2], NULL);

   ud = ibfind(udname);
   
   if (gpibError(interp, resultPtr, "ibfind") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ud);
   return TCL_OK;
}

int iblinesCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, option;
   short lines;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   iblines(ud, &lines);
   if (gpibError(interp, resultPtr, "iblines") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, lines);
   return TCL_OK;
}

int iblloCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   ibllo(ud);
   if (gpibError(interp, resultPtr, "ibllo") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);
   return TCL_OK;
}

int iblnCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, pad, sad;
   short listen;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 5)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud pad sad");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &pad) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[4], &sad) != TCL_OK)
      return TCL_ERROR;

   ud = ibln(ud, pad, sad, &listen);
   if (gpibError(interp, resultPtr, "ibln") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, listen);
   return TCL_OK;
}

int iblocCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   ibloc(ud);
   if (gpibError(interp, resultPtr, "ibloc") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);
   return TCL_OK;
}

int ibonlCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetBooleanFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibonl(ud, v);
   if (gpibError(interp, resultPtr, "ibonl") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibpadCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibpad(ud, v);
   if (gpibError(interp, resultPtr, "ibpad") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibpctCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   ibpct(ud);
   if (gpibError(interp, resultPtr, "ibpct") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibpokeCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, option, value;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 5)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &option) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[4], &value) != TCL_OK)
      return TCL_ERROR;

   ibpoke(ud, option, value);
   if (gpibError(interp, resultPtr, "ibpoke") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibppcCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibppc(ud, v);
   if (gpibError(interp, resultPtr, "ibppc") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibrdCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   char *buf;
   int ud;
   long count;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud, count");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetLongFromObj(interp, objv[3], &count) != TCL_OK)
      return TCL_ERROR;

   if ( (buf = (char *)malloc(count)) == NULL)  {
      Tcl_SetStringObj(resultPtr, "ibrd malloc failure", -1);
      return TCL_ERROR;
   }

   /* Let's first wait for the interface to be clear */
   ibwait(ud, CMPL);
   if (gpibError(interp, resultPtr, "Waiting for ibrd") != TCL_OK)  {
      free(buf);
      return TCL_ERROR;
   }

   ibrd(ud, buf, count);

   if (gpibError(interp, resultPtr, "ibrd") != TCL_OK)  {
      free(buf);
      return TCL_ERROR;
   }

   Tcl_SetStringObj(resultPtr, buf, ibcntl);   /* return read data */
   free(buf);
   return TCL_OK;
}


int ibrdfCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   char *filename;
   int ud;
   long count;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud, filename");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   filename = Tcl_GetStringFromObj(objv[3], NULL);

   /* Let's first wait for the interface to be clear */
   ibwait(ud, CMPL);
   if (gpibError(interp, resultPtr, "Waiting for ibrd") != TCL_OK)  {
      return TCL_ERROR;
   }

   ibrdf(ud, filename);

   if (gpibError(interp, resultPtr, "ibrdf") != TCL_OK)  {
      return TCL_ERROR;
   }

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}


int ibrppCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;
   char v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   ibrpp(ud, &v);
   if (gpibError(interp, resultPtr, "ibrpp") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, v);   /* return status */
   return TCL_OK;
}

int ibrscCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetBooleanFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibrsc(ud, v);
   if (gpibError(interp, resultPtr, "ibrsc") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibrspCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;
   char v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   ibrsp(ud, &v);
   if (gpibError(interp, resultPtr, "ibrsp") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, v);   /* return status */
   return TCL_OK;
}

int ibrsvCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibrsv(ud, v);
   if (gpibError(interp, resultPtr, "ibrsv") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibsadCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibsad(ud, v);
   if (gpibError(interp, resultPtr, "ibsad") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibsicCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   ibsic(ud);
   if (gpibError(interp, resultPtr, "ibsic") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibsreCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ibsre ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetBooleanFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibsre(ud, v);
   if (gpibError(interp, resultPtr, "ibsre") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, iberr);   /* return prev value */
   return TCL_OK;
}

/**************************************************************
void ibsrqCallback(void);

int ibsrqCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "Tcl_proc_name");
      return TCL_ERROR;
   }
   ibsrq( (void *) *ibsrqCallback);
   if (gpibError(interp, resultPtr, "ibstop") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   
   return TCL_OK;
}

void ibsrqCallback(void)
{
   printf ("Got into SRQ callback!!\n");
}
**************************************************************/
int ibstopCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   ibstop(ud);
   if (gpibError(interp, resultPtr, "ibstop") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibtmoCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud v");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &v) != TCL_OK)
      return TCL_ERROR;

   ibtmo(ud, v);
   if (gpibError(interp, resultPtr, "ibtmo") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

/* As far as I can tell, there is no ibtrap...
int ibtrapCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int mask, mode;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "mask mode");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &mask) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &mode) != TCL_OK)
      return TCL_ERROR;

   ibtrap(mask, mode);
   if (gpibError(interp, resultPtr, "ibtrap") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   
   return TCL_OK;
}
*/

int ibtrgCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, v;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   ibtrg(ud);
   if (gpibError(interp, resultPtr, "ibtrg") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibwaitCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, mask;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ud mask");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &mask) != TCL_OK)
      return TCL_ERROR;

   ibwait(ud, mask);
   if (gpibError(interp, resultPtr, "ibwait") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibwrtCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud, count;
   char *buf;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ibwrt ud, wrtbuf");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   buf = Tcl_GetStringFromObj(objv[3], &count);

   if ( debug & DATA_WRITE)
      printf ("ibwrt: ud %d, length %d, data %s\n", ud, count, buf);

   ibwrt(ud, buf, count);
   if (gpibError(interp, resultPtr, "ibwrt") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int ibwrtfCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int ud;
   char *filename;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ibwrt ud, filename");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &ud) != TCL_OK)
      return TCL_ERROR;

   filename = Tcl_GetStringFromObj(objv[3], NULL);

   if ( debug & DATA_WRITE)
      printf ("ibwrtf: ud %d, filename: %s\n", ud, filename);

   ibwrtf(ud, filename);
   if (gpibError(interp, resultPtr, "ibwrtf") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

/************************************************************************/
/* 488.2 functions							*/
/************************************************************************/
#define MAX_ADDR 32+1


int AllSpollCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr, **listPtr;
   int boardID, count, i, temp;
   Addr4882_t addrlist[MAX_ADDR];
   short resultlist[MAX_ADDR];

   resultPtr = Tcl_GetObjResult(interp);
   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "AllSpoll boardID, addrlist");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_ListObjGetElements(interp, objv[3], &count, &listPtr) != TCL_OK)
      return TCL_ERROR;

   for (i=0; i<count; i++)  {
      if (Tcl_GetIntFromObj(interp, *listPtr++, &temp) != TCL_OK) {
         Tcl_SetStringObj(resultPtr, "Tcl_GetIntFromObj in AllSpoll", -1);
         return TCL_ERROR;
      }
      addrlist[i] = temp;
   }
   addrlist[i] = NOADDR;

   AllSpoll(boardID, addrlist, resultlist);

   if (gpibError(interp, resultPtr, "AllSpoll") != TCL_OK)
      return TCL_ERROR;

   for (i=0; i < ibcntl; i++)  {
      if (Tcl_ListObjAppendElement(interp, resultPtr, Tcl_NewIntObj(resultlist[i]) ) != TCL_OK)
         return TCL_ERROR;
   }
   return TCL_OK;
}

int DevClearCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int boardID, temp;
   Addr4882_t address;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "boardID address");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &temp) != TCL_OK)
      return TCL_ERROR;

   address = temp;
   DevClear(boardID, address);
   if (gpibError(interp, resultPtr, "DevClear") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int DevClearListCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr, **listPtr;
   int boardID, count, i, temp;
   Addr4882_t addrlist[MAX_ADDR];
   short resultlist[MAX_ADDR];

   resultPtr = Tcl_GetObjResult(interp);
   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "DevClearList boardID, addrlist");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_ListObjGetElements(interp, objv[3], &count, &listPtr) != TCL_OK)
      return TCL_ERROR;

   for (i=0; i<count; i++)  {
      if (Tcl_GetIntFromObj(interp, *listPtr++, &temp) != TCL_OK) {
         Tcl_SetStringObj(resultPtr, "Tcl_GetIntFromObj in DevClearList", -1);
         return TCL_ERROR;
      }
      addrlist[i] = temp;
   }
   addrlist[i] = NOADDR;

   DevClearList(boardID, addrlist);

   if (gpibError(interp, resultPtr, "DevClearList") != TCL_OK)
      return TCL_ERROR;

   return TCL_OK;
}

int EnableLocalCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr, **listPtr;
   int boardID, count, i, temp;
   Addr4882_t addrlist[MAX_ADDR];

   resultPtr = Tcl_GetObjResult(interp);
   if (objc =! 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "EnableLocal boardID, addrlist");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_ListObjGetElements(interp, objv[3], &count, &listPtr) != TCL_OK)
      return TCL_ERROR;

   for (i=0; i<count; i++)  {
      if (Tcl_GetIntFromObj(interp, *listPtr++, &temp) != TCL_OK) {
         Tcl_SetStringObj(resultPtr, "Tcl_GetIntFromObj in EnableLocal", -1);
         return TCL_ERROR;
      }
      addrlist[i] = temp;
   }
   addrlist[i] = NOADDR;

   EnableLocal(boardID, addrlist);

   if (gpibError(interp, resultPtr, "EnableLocal") != TCL_OK)
      return TCL_ERROR;

   return TCL_OK;
}

int EnableRemoteCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr, **listPtr;
   int boardID, count, i, temp;
   Addr4882_t addrlist[MAX_ADDR];

   resultPtr = Tcl_GetObjResult(interp);
   if (objc =! 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "EnableRemote boardID, addrlist");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_ListObjGetElements(interp, objv[3], &count, &listPtr) != TCL_OK)
      return TCL_ERROR;

   for (i=0; i<count; i++)  {
      if (Tcl_GetIntFromObj(interp, *listPtr++, &temp) != TCL_OK) {
         Tcl_SetStringObj(resultPtr, "Tcl_GetIntFromObj in EnableRemote", -1);
         return TCL_ERROR;
      }
      addrlist[i] = temp;
   }
   addrlist[i] = NOADDR;

   EnableRemote(boardID, addrlist);

   if (gpibError(interp, resultPtr, "EnableRemote") != TCL_OK)
      return TCL_ERROR;

   return TCL_OK;
}

int FindLstnCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr, **listPtr;
   int boardID, count, i, temp, limit;
   Addr4882_t addrlist[MAX_ADDR];
   short resultlist[MAX_ADDR];

   resultPtr = Tcl_GetObjResult(interp);
   if (objc < 4 && objc > 5)  {
      Tcl_WrongNumArgs(interp, 1, objv, "FindLstn boardID, addrlist ?limit?");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_ListObjGetElements(interp, objv[3], &count, &listPtr) != TCL_OK)
      return TCL_ERROR;

   if (objc == 5)  {
      if (Tcl_GetIntFromObj(interp, objv[4], &limit) != TCL_OK)
         return TCL_ERROR;
   }  else  {
      limit = count;
   }

   for (i=0; i<count; i++)  {
      if (Tcl_GetIntFromObj(interp, *listPtr++, &temp) != TCL_OK) {
         Tcl_SetStringObj(resultPtr, "Tcl_GetIntFromObj in FindLstnCmd", -1);
         return TCL_ERROR;
      }
      addrlist[i] = temp;
   }
   addrlist[i] = NOADDR;

   FindLstn(boardID, addrlist, resultlist, limit);

   if (gpibError(interp, resultPtr, "FindLstn") != TCL_OK)
      return TCL_ERROR;

   for (i=0; i < ibcntl; i++)  {
      if (Tcl_ListObjAppendElement(interp, resultPtr, Tcl_NewIntObj(resultlist[i]) ) != TCL_OK)
         return TCL_ERROR;
   }
   return TCL_OK;
}

int FindRQSCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr, **listPtr;
   int boardID, count, i, temp;
   Addr4882_t addrlist[MAX_ADDR];
   short result;

   resultPtr = Tcl_GetObjResult(interp);
   if (objc =! 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "FindRQS boardID, addrlist");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_ListObjGetElements(interp, objv[3], &count, &listPtr) != TCL_OK)
      return TCL_ERROR;

   for (i=0; i<count; i++)  {
      if (Tcl_GetIntFromObj(interp, *listPtr++, &temp) != TCL_OK) {
         Tcl_SetStringObj(resultPtr, "Tcl_GetIntFromObj in FindRQS", -1);
         return TCL_ERROR;
      }
      addrlist[i] = temp;
   }
   addrlist[i] = NOADDR;

   FindRQS(boardID, addrlist, &result);

   if (gpibError(interp, resultPtr, "FindRQS") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, result);

   return TCL_OK;
}

int PPollCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int boardID;
   short result;

   resultPtr = Tcl_GetObjResult(interp);
   if (objc =! 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "PPoll boardID");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   PPoll(boardID, &result);

   if (gpibError(interp, resultPtr, "PPoll") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, result);

   return TCL_OK;
}

int PPollConfigCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int boardID, temp, dataLine, lineSense;
   Addr4882_t address;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 6)  {
      Tcl_WrongNumArgs(interp, 1, objv, "boardID address dataLine lineSense");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &temp) != TCL_OK)
      return TCL_ERROR;
   address = temp;

   if (Tcl_GetIntFromObj(interp, objv[4], &dataLine) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetBooleanFromObj(interp, objv[5], &lineSense) != TCL_OK)
      return TCL_ERROR;

   PPollConfig(boardID, address, dataLine, lineSense);
   if (gpibError(interp, resultPtr, "PPollConfig") != TCL_OK)
      return TCL_ERROR;

   return TCL_OK;
}


int PPollUnconfigCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr, **listPtr;
   int boardID, count, i, temp;
   Addr4882_t addrlist[MAX_ADDR];

   resultPtr = Tcl_GetObjResult(interp);
   if (objc =! 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "PPollUnconfig boardID, addrlist");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_ListObjGetElements(interp, objv[3], &count, &listPtr) != TCL_OK)
      return TCL_ERROR;

   for (i=0; i<count; i++)  {
      if (Tcl_GetIntFromObj(interp, *listPtr++, &temp) != TCL_OK) {
         Tcl_SetStringObj(resultPtr, "Tcl_GetIntFromObj in PPollUnconfig", -1);
         return TCL_ERROR;
      }
      addrlist[i] = temp;
   }
   addrlist[i] = NOADDR;

   PPollUnconfig(boardID, addrlist);

   if (gpibError(interp, resultPtr, "PPollUnconfig") != TCL_OK)
      return TCL_ERROR;

   return TCL_OK;
}


int PassControlCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int boardID, temp;
   Addr4882_t address;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "boardID address");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &temp) != TCL_OK)
      return TCL_ERROR;

   address = temp;
   PassControl(boardID, address);
   if (gpibError(interp, resultPtr, "PassControl") != TCL_OK)
      return TCL_ERROR;

   return TCL_OK;
}


int RcvRespMsgCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   char *buf;
   int boardID, termination;
   long count;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 5)  {
      Tcl_WrongNumArgs(interp, 1, objv, "boardID, count, termination");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetLongFromObj(interp, objv[3], &count) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[4], &termination) != TCL_OK)
      return TCL_ERROR;

   if ( (buf = (char *)malloc(count)) == NULL)  {
      Tcl_SetStringObj(resultPtr, "RevRespMsg malloc failure", -1);
      return TCL_ERROR;
   }

   RcvRespMsg(boardID, buf, count, termination);

   if (gpibError(interp, resultPtr, "RcvRespMsg") != TCL_OK)  {
      free(buf);
      return TCL_ERROR;
   }

   Tcl_SetStringObj(resultPtr, buf, ibcntl);   /* return read data */
   free(buf);
   return TCL_OK;
}

int ReadStatusByteCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int boardID, temp;
   Addr4882_t address;
   short result;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "boardID address");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &temp) != TCL_OK)
      return TCL_ERROR;
   address = temp;

   ReadStatusByte(boardID, address, &result);
   if (gpibError(interp, resultPtr, "ReadStatusByte") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, result);

   return TCL_OK;
}

int ReceiveCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   Addr4882_t address;
   char *buf;
   int boardID, termination, temp;
   long count;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 6)  {
      Tcl_WrongNumArgs(interp, 1, objv, "boardID, address, count, termination");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &temp) != TCL_OK)
      return TCL_ERROR;
   address = temp;

   if (Tcl_GetLongFromObj(interp, objv[4], &count) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[5], &termination) != TCL_OK)
      return TCL_ERROR;

   if ( (buf = (char *)malloc(count)) == NULL)  {
      Tcl_SetStringObj(resultPtr, "Receive malloc failure", -1);
      return TCL_ERROR;
   }

   Receive(boardID, address, buf, count, termination);

   if (gpibError(interp, resultPtr, "Receive") != TCL_OK)  {
      free(buf);
      return TCL_ERROR;
   }

   Tcl_SetStringObj(resultPtr, buf, ibcntl);   /* return read data */
   free(buf);
   return TCL_OK;
}

int ReceiveSetupCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int boardID, temp;
   Addr4882_t address;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "boardID address");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &temp) != TCL_OK)
      return TCL_ERROR;
   address = temp;

   ReceiveSetup(boardID, address);
   if (gpibError(interp, resultPtr, "ReceiveSetup") != TCL_OK)
      return TCL_ERROR;

   return TCL_OK;
}

int ResetSysCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr, **listPtr;
   int boardID, count, i, temp;
   Addr4882_t addrlist[MAX_ADDR];

   resultPtr = Tcl_GetObjResult(interp);
   if (objc =! 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "ResetSys boardID, addrlist");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_ListObjGetElements(interp, objv[3], &count, &listPtr) != TCL_OK)
      return TCL_ERROR;

   for (i=0; i<count; i++)  {
      if (Tcl_GetIntFromObj(interp, *listPtr++, &temp) != TCL_OK) {
         Tcl_SetStringObj(resultPtr, "Tcl_GetIntFromObj in ResetSys", -1);
         return TCL_ERROR;
      }
      addrlist[i] = temp;
   }
   addrlist[i] = NOADDR;

   ResetSys(boardID, addrlist);

   if (gpibError(interp, resultPtr, "ResetSys") != TCL_OK)
      return TCL_ERROR;

   return TCL_OK;
}

int SendCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   Addr4882_t address;
   char *buf;
   int boardID, eotmode, temp;
   long datacnt;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 6)  {
      Tcl_WrongNumArgs(interp, 1, objv, "boardID, address, buf, eotmode");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetIntFromObj(interp, objv[3], &temp) != TCL_OK)
      return TCL_ERROR;
   address = temp;

   buf = Tcl_GetStringFromObj(objv[4], &temp);
   if (buf == NULL)  {
      Tcl_SetStringObj(resultPtr, "Send: Invalid data string", -1);
      return TCL_ERROR;
   }
   datacnt = temp;

   if (Tcl_GetIntFromObj(interp, objv[5], &eotmode) != TCL_OK)
      return TCL_ERROR;

   Send (boardID, address, buf, datacnt, eotmode);

   if (gpibError(interp, resultPtr, "Send") != TCL_OK)  
      return TCL_ERROR;

   return TCL_OK;
}

int SendCmdsCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   char *buf;
   int boardID, temp;
   long datacnt;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 4)  {
      Tcl_WrongNumArgs(interp, 1, objv, "boardID, buf");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   buf = Tcl_GetStringFromObj(objv[3], &temp);
   if (buf == NULL)  {
      Tcl_SetStringObj(resultPtr, "SendCmds: Invalid data string", -1);
      return TCL_ERROR;
   }
   datacnt = temp;

   SendCmds (boardID, buf, datacnt);

   if (gpibError(interp, resultPtr, "SendCmds") != TCL_OK)  
      return TCL_ERROR;

   return TCL_OK;
}

int SendDataBytesCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   char *buf;
   int boardID, eotmode, temp;
   long datacnt;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 5)  {
      Tcl_WrongNumArgs(interp, 1, objv, "boardID, buf, eotmode");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   buf = Tcl_GetStringFromObj(objv[3], &temp);
   if (buf == NULL)  {
      Tcl_SetStringObj(resultPtr, "SendDataBytes: Invalid data string", -1);
      return TCL_ERROR;
   }
   datacnt = temp;

   if (Tcl_GetIntFromObj(interp, objv[4], &eotmode) != TCL_OK)
      return TCL_ERROR;

   SendDataBytes (boardID, buf, datacnt, eotmode);

   if (gpibError(interp, resultPtr, "SendDataBytes") != TCL_OK)  
      return TCL_ERROR;

   return TCL_OK;
}

int SendIFCCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int boardID;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "boardID");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   SendIFC(boardID);
   if (gpibError(interp, resultPtr, "SendIFC") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int SendListCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr, **listPtr;
   int boardID, count, i, temp, eotmode;
   Addr4882_t addrlist[MAX_ADDR];
   long datacnt;
   char *buf;

   resultPtr = Tcl_GetObjResult(interp);
   if (objc != 6)  {
      Tcl_WrongNumArgs(interp, 1, objv, "SendList boardID, addrlist buffer eotmode");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_ListObjGetElements(interp, objv[3], &count, &listPtr) != TCL_OK)
      return TCL_ERROR;

   if (Tcl_GetStringFromObj(objv[4], &temp) != NULL)  {
      Tcl_SetStringObj(resultPtr, "SendList: Invalid data string", -1);
      return TCL_ERROR;
   }
   datacnt = temp;

   if (Tcl_GetIntFromObj(interp, objv[5], &eotmode) != TCL_OK)
      return TCL_ERROR;

   for (i=0; i<count; i++)  {
      if (Tcl_GetIntFromObj(interp, *listPtr++, &temp) != TCL_OK) {
         Tcl_SetStringObj(resultPtr, "Tcl_GetIntFromObj in SendList", -1);
         return TCL_ERROR;
      }
      addrlist[i] = temp;
   }
   addrlist[i] = NOADDR;

   SendList(boardID, addrlist, buf, datacnt, eotmode);

   if (gpibError(interp, resultPtr, "SendList") != TCL_OK)
      return TCL_ERROR;

   return TCL_OK;
}

int SendLLOCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int boardID;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc != 3)  {
      Tcl_WrongNumArgs(interp, 1, objv, "boardID");
      return TCL_ERROR;
   }
   if (Tcl_GetIntFromObj(interp, objv[2], &boardID) != TCL_OK)
      return TCL_ERROR;

   SendLLO(boardID);
   if (gpibError(interp, resultPtr, "SendLLO") != TCL_OK)
      return TCL_ERROR;

   Tcl_SetIntObj(resultPtr, ibsta);   /* return status */
   return TCL_OK;
}

int MapStatusToAsciiCmd (ClientData clientData, Tcl_Interp *interp,
    int objc, Tcl_Obj *CONST objv[])
{
   Tcl_Obj *resultPtr;
   int status;

   resultPtr = Tcl_GetObjResult(interp);

   if (objc > 3) {
      Tcl_WrongNumArgs(interp, 1, objv, "?status?");
      return TCL_ERROR;
   }

   if (objc == 2)  {
      if (Tcl_GetIntFromObj(interp, objv[2], &status) != TCL_OK)
         return TCL_ERROR;
   } else {
      status = ibsta;
   }
   Tcl_FreeResult(interp);
   Mapstatustoascii(resultPtr, status);
   return TCL_OK;
}

/************************************************************************/
/* "helper" functions, not callable from Tcl                            */
/************************************************************************/

/* find comand function pointer in hash table */
/* stolen from mx */
int (*CmdFind(char *string))()
{
   Tcl_HashEntry *entryPtr;
   int (*handler) ();
   entryPtr = Tcl_FindHashEntry(&cmdTable, string);
   if (entryPtr == NULL)  {
      return NULL;
   }
   handler = (int(*)())Tcl_GetHashValue(entryPtr);
   return handler;
}

int gpibError (Tcl_Interp *interp, Tcl_Obj *resultPtr, char *string)
{
   Tcl_HashEntry *entryPtr;

   if (ibsta & ERR)  {
      Tcl_SetStringObj(resultPtr, string, -1);
      Tcl_AppendToObj(resultPtr, " errors: ", -1);
      entryPtr = Tcl_FindHashEntry(&errTable, (char *)iberr);
      if (entryPtr == NULL)
         Tcl_AppendToObj(resultPtr, "unknown error code", -1);
      else
         Tcl_AppendToObj(resultPtr, (char *)Tcl_GetHashValue(entryPtr), -1);

   Tcl_AppendToObj(resultPtr, " Status:", -1);
   Mapstatustoascii(resultPtr, ibsta);
   return TCL_ERROR;
   }
 return TCL_OK;
}

void Mapstatustoascii (Tcl_Obj *resultPtr, int status)
{
   if (status & ERR)
      Tcl_AppendToObj(resultPtr, "ERR ", -1);
   if (status & TIMO)
      Tcl_AppendToObj(resultPtr, "TIMO ", -1);
   if (status & END)
      Tcl_AppendToObj(resultPtr, "END ", -1);
   if (status & SRQI)
      Tcl_AppendToObj(resultPtr, "SRQI ", -1);
   if (status & RQS)
      Tcl_AppendToObj(resultPtr, "RQS ", -1);
   if (status & SPOLL)
      Tcl_AppendToObj(resultPtr, "SPOLL ", -1);
   if (status & EVENT)
      Tcl_AppendToObj(resultPtr, "EVENT ", -1);
   if (status & CMPL)
      Tcl_AppendToObj(resultPtr, "CMPL ", -1);
   if (status & LOK)
      Tcl_AppendToObj(resultPtr, "LOK ", -1);
   if (status & REM)
      Tcl_AppendToObj(resultPtr, "REM ", -1);
   if (status & CIC)
      Tcl_AppendToObj(resultPtr, "CIC ", -1);
   if (status & ATN)
      Tcl_AppendToObj(resultPtr, "ATN ", -1);
   if (status & TACS)
      Tcl_AppendToObj(resultPtr, "TACS ", -1);
   if (status & LACS)
      Tcl_AppendToObj(resultPtr, "LACS ", -1);
   if (status & DTAS)
      Tcl_AppendToObj(resultPtr, "DTAS ", -1);
   if (status & DCAS)
      Tcl_AppendToObj(resultPtr, "DCAS ", -1);
}

